from ContainerTarget import ContainerTarget
from utils.TilingImage import TilingImage
from utils.Hash2D import Hash2D
from utils.datatypes import *
from utils import vfs

import gtk
import os



#
# Class for grouping different targets together.
#
class TargetGroup(ContainerTarget):

    def __init__(self, parent, display):

        # the area zones covered by the child targets
        self.__zones = Hash2D()

        # the background pixmap
        self.__pixbuf = None

        # flag for ensuring that only one collapse action takes place
        self.__block = 0
        
        
        ContainerTarget.__init__(self, parent, display)
        self.__layout = gtk.Fixed()
        self.__layout.show()

        self.add(self.__layout)

        self.__image = TilingImage()
        self.__image.show()
        self.__layout.put(self.__image, 0, 0)

        self._set_property_type("bg-uri", TYPE_STRING)
        self._set_property_type("bg-color", TYPE_STRING)

        self.add_observer(self.__on_observe_size)




    def __on_observe_size(self, src, cmd, *data):

        if (cmd == src.OBS_MOVE):
            x, y, w, h = data
            self.__image.set_size_request(w, h)
            self.__image.tile()



    def add_children(self, childrendata):

        ContainerTarget.add_children(self, childrendata)
        for child in self._get_children():
            self.__layout.put(child, 0, 0)
            self.__zones.set(0, 0, 0, 0, child)
            child.add_observer(self.__on_observe_target)
        #end for

        self._init_children()



    def __on_observe_target(self, src, cmd, *args):

        if (cmd == src.OBS_MOVE):
            x, y, w, h = args
            x, y = src.get_anchored_coords(x, y, w, h)
            self.__layout.move(src, x, y)
            self.__zones.set(x, y, x + w, y + h, src)

            # give the group a chance to shrink
            self.__collapse_bg()



    #
    # Makes the background collapse so that the group can shrink.
    #
    def __collapse_bg(self):

        if (self.__block): return
        self.__block = 1
        #self.__image.hide()
        self.__image.set_size_request(-1, -1)
        #gtk.idle_add(self.__image.show)

        def x(self): self.__block = 0
        gtk.timeout_add(500, x, self)


    #
    # Sets the background color.
    #
    def __set_color(self, color):

        self.__image.set_from_color(color)


    #
    # Sets the background image.
    #
    def __set_background(self, uri):

        if (not uri):
            self.__set_color("#00000000")
            return

        uri = os.path.join(self.get_path(), uri)
        if (vfs.exists(uri)):
            self.__image.set_from_file(uri)
            
        else:
            self.__set_color("#ffffffa0")

        #self.__image.set_size_request(width, height)



    def set_config(self, key, value):

        value = self._convert_type(key, value)

        if (key == "bg-uri"):
            self.__set_background(value)
            
        elif (key == "bg-color"):
            self.__set_color(value)

        else:
            ContainerTarget.set_config(self, key, value)



    def get_target_at(self, px, py, path, is_root = 0):

        x, y, w, h = self.get_geometry()
        x, y = self.get_anchored_coords(x, y, w, h)

        if (is_root): x, y = 0, 0
        if (x <= px <= x + w and y <= py <= y + h):
            targets = self.__zones.get(px - x, py - y)
            ctargets = []
            
            children = self._get_children()
            children.reverse()
            for t in children:
                if (t in targets):
                    ctargets += t.get_target_at(px - x , py - y, path)
                    #if (ctargets): break
            #end for

            ret = [(self, path)] + ctargets
            return ret

        else:
            return []
        
        #end if
